indexing
	description: "This class represents an alien in the game."
	author: "Lukas Novosad - novosprotos@eblcom.ch"
	date: "$Date: 2005-06-15 11:55:15 +0200 (Mi, 15 Jun 2005) $"
	revision: "$Revision: 0.8 $"

class
	ALIEN

inherit

	DR_SPRITE_OBJECT
	redefine
		make_from_animation
	end

create
	make_from_animation


feature {NONE} -- Creation

	make_from_animation (an_animation: EM_ANIMATION; an_x, a_y: INTEGER) is
			-- Create the ALIEN object.
		do
			precursor (an_animation, an_x, a_y)
			set_do_loop (true)
			start
			set_state (NONE)
			image_offset_x := 0.0
			image_offset_y := 0.0
			-- Call create from EM_RECTANGLE_COLLIDABLE to ensure the logger gets created (EM_LOGGING class invariant).
			make_from_rectangle(bounding_box)
		end


feature -- Setter methods

	set_state (i:INTEGER) is
			-- Set the current state.
		do
			state := i
		ensure
			state_set: state = i
		end


	set_direction (a_dir: EM_VECTOR_2D) is
			-- Set the direction the alien is moving into.
		require a_dir_not_void: a_dir /= void
		do
			direction := a_dir
		ensure
			direction_set: direction = a_dir
		end


--	remember_position is
--			-- Remember the current position.
--		do
--			last_position := last_center
--		ensure
--			last_position_set: last_position = last_center
--		end


feature -- Status

	type_id: INTEGER is
			-- The id given to all instances of `current' generating class.
		do
			result := ALIEN_ID
		end

feature -- Collision handling

	on_collide (other: DR_OBJECT; collision: EM_COLLISION [DR_OBJECT]; frame_delay: INTEGER) is
			-- `current' collided with `other'
		local
			time_1, time_2: DOUBLE
			dir_1, dir_2, pos_1, pos_2: EM_VECTOR_2D
		do
			if other.type_id = DRAGON_ID then
				-- go to trap state
				set_state (DEAD)

			elseif other.type_id = STONE_BORDER_HORIZONTAL_ID or other.type_id = STONE_BORDER_VERTICAL_ID or other.type_id = STONE_FIXED_ID or other.type_id = STONE_MOVING_ID or other.type_id = CHEST_ID or other.type_id = EXIT_ID then

				if current.state = NONE then

					current.push_back (other)

				else

					dir_1 := current.direction

						-- update state
					if current.state = UP then
						current.set_state (DOWN)
						current.set_direction (create {EM_VECTOR_2D}.make (0.0, 1.0))
					elseif current.state = DOWN then
						current.set_state (UP)
						current.set_direction (create {EM_VECTOR_2D}.make (0.0, -1.0))
					elseif current.state = LEFT then
						current.set_state (RIGHT)
						current.set_direction (create {EM_VECTOR_2D}.make (1.0, 0.0))
					elseif current.state = RIGHT then
						current.set_state (LEFT)
						current.set_direction (create {EM_VECTOR_2D}.make (-1.0, 0.0))
					end

					dir_2 := current.direction

					time_1 := collision.time
					--dir_1 := current.direction
					--io.put_string (time.ticks.out + "ALIEN: direction before collision is " + dir_1.out)
					--io.new_line
					--dir_2 := other.direction
					--io.new_line
					--io.put_string (time.ticks.out + "ALIEN: collision_direction is " + collision.collision_direction.to_vector.out)
					--dir_1 := dir_1.reflection (collision.collision_direction.to_vector)
					--io.put_string (time.ticks.out + "ALIEN: direction after collision is " + dir_1.out)
					--io.new_line
					--dir_2 := dir_2.reflection (collision.collision_direction.to_vector)
					--current.set_direction (dir_1)

					-- if the collision time is not 1.0 depth search was performed and the position must be updated accordingly
					if time_1 < 1.0 and time_1 > 0.0 then
						--io.put_string (time.ticks.out + "ALIEN: collision time is smaller than 1.0: " + time_1.out)
						--io.new_line
						time_2 := (1.0 - time_1) * frame_delay / 1000
						time_1 := time_1 * frame_delay / 1000

						-- update position of current
						pos_1 := dir_1 * time_1
						pos_2 := dir_2 * time_2
						current.set_center (current.last_center + pos_1 + pos_2)

						-- update position of other
						--pos_1 := other.direction * time_1
						--pos_2 := dir_2 * time_2
						--other.set_center (other.last_center + pos_1 + pos_2)
					elseif time_1 = 1.0 then
						--io.put_string (time.ticks.out + "ALIEN: collision time is equal to 1.0: " + time_1.out)
						--io.new_line
						time_1 := time_1 * frame_delay / 1000

						-- update position of current
						pos_1 := dir_2 * time_1
						current.set_center (current.last_center + pos_1)

					elseif time_1 = 0.0 then
						--io.put_string (time.ticks.out + "ALIEN: collision time is equal to 0.0: " + time_1.out)
						--io.new_line
						time_1 := 1.0 * frame_delay / 1000

						-- update position of current
						pos_1 := dir_1 * time_1
						current.set_center (current.last_center + pos_1)

					end

				end

					-- update image position
				set_x_y ( (corner_top_left.x - image_offset_x).floor, (corner_top_left.y - image_offset_y).floor )

			end

		end


feature -- Commands

	step_by (a_delay: DOUBLE) is
			-- Move the alien based on its current direction and the time delay.
		local
			movement: EM_VECTOR_2D
		do
			create movement.make_from_other (direction)
			movement.scale_to (Delta_move)
			movement.scale (a_delay / 1000)
			--set_center (center + (direction*(a_delay/1000)) )
			--create last_position.make_from_other ( last_center )
			move_by ( movement.x, movement.y )
			set_x_y ( (corner_top_left.x - image_offset_x).floor, (corner_top_left.y - image_offset_y).floor )
		end


feature -- Implementation

	state: INTEGER
		-- The state the alien is in, see ZLEVEL -> Constants.

	direction: EM_VECTOR_2D
		-- The direction the alien is currently moving into.

	--last_position: EM_VECTOR_2D
		-- The position before the last call of step_by.


end -- class ALIEN
