indexing
	description: "Objects that represent game objects in Dragoony."
	author: "Lukas Novosad - novosprotos@eblcom.ch"
	date: "$Date: 2012-01-15 $"
	revision: "$Revision: 0.8 $"

deferred class
	DR_OBJECT

inherit

	EM_RECTANGLE_COLLIDABLE
	rename
		make as make_2,
		width as width_2,
		height as height_2,
		set_x_y as set_x_y_2,
		set_width as set_width_2,
		set_height as set_height_2

	undefine
		draw
	end

	DR_CONSTANTS


feature -- Status

	type_id: INTEGER is
			-- The id given to all instances of `current' generating class.
		deferred
		end


feature -- Collision handling

	on_collide (other: DR_OBJECT; collision: EM_COLLISION [DR_OBJECT]; frame_delay: INTEGER) is
			-- the collsion handling subroutine
		deferred
		end

	push_back (other: DR_OBJECT)
			-- Simply push back `current' against the boundary of `other'.
			-- TODO: Implement cases to handle complete inclusions.
		local
			margin_x, margin_y: REAL
		do
			margin_x := 0.0
			margin_y := 0.0

			if current.border_left >= other.border_left and current.border_right <= other.border_right or current.border_left <= other.border_left and current.border_right >= other.border_right then
				-- current is either on top or below, no corners involved

				if current.border_bottom > other.border_top and current.border_bottom < other.border_bottom then
					-- on top
					center.set_y (other.border_top - current.half_height - 0.0001)
					--set_x_y_2 (center.x, other.border_top - current.half_height - 0.0001)
				elseif current.border_top > other.border_top and current.border_top < other.border_bottom then
					-- below
					center.set_y (other.border_bottom + current.half_height + 0.0001)
					--set_x_y_2 (center.x, other.border_bottom + current.half_height + 0.0001)
				end

			elseif current.border_top >= other.border_top and current.border_bottom <= other.border_bottom or current.border_top <= other.border_top and current.border_bottom >= other.border_bottom then
				-- current is either on the left or on the right side, no corners involved

				if current.border_left < other.border_right and current.border_left > other.border_left then
					-- to the right
					center.set_x (other.border_right + current.half_width + 0.0001)
					--set_x_y_2 (other.border_right + current.half_width + 0.0001, center.y)

				elseif current.border_right > other.border_left and current.border_right < other.border_right then
					-- to the left
					center.set_x (other.border_left - current.half_width - 0.0001)
					--set_x_y_2 (other.border_left - current.half_width - 0.0001, center.y)
				end

			elseif current.border_left < other.border_left and current.border_right > other.border_left then
				-- left corners
				if current.border_bottom > other.border_top and current.border_bottom < other.border_bottom then
					-- upper left corner
					margin_x := current.corner_bottom_right.x - other.border_left
					margin_y := current.corner_bottom_right.y - other.border_top
					if margin_x <= margin_y then
						center.set_x (other.border_left - current.half_width - 0.0001)
						--set_x_y_2 (other.border_left - current.half_width - 0.0001, center.y)
					else
						center.set_y (other.border_top - current.half_height - 0.0001)
						--set_x_y_2 (center.x, other.border_top - current.half_height - 0.0001)
					end

				elseif current.border_top > other.border_top and current.border_top < other.border_bottom then
					-- lower left corner
					margin_x := current.corner_top_right.x - other.border_left
					margin_y := other.border_bottom - current.corner_top_right.y
					if margin_x <= margin_y then
						center.set_x (other.border_left - current.half_width - 0.0001)
						--set_x_y_2 (other.border_left - current.half_width - 0.0001, center.y)
					else
						center.set_y (other.border_bottom + current.half_height + 0.0001)
						--set_x_y_2 (center.x, other.border_bottom + current.half_height + 0.0001)
					end

				end

			elseif current.border_left < other.border_right and current.border_right > other.border_right then
				-- right corners
				if current.border_bottom > other.border_top and current.border_bottom < other.border_bottom then
					-- upper right corner
					margin_x := other.border_right - current.corner_bottom_left.x
					margin_y := current.corner_bottom_left.y - other.border_top
					if margin_x <= margin_y then
						center.set_x (other.border_right + current.half_width + 0.0001)
						--set_x_y_2 (other.border_right + current.half_width + 0.0001, center.y)
					else
						center.set_y (other.border_top - current.half_height - 0.0001)
						--set_x_y_2 (center.x, other.border_top - current.half_height - 0.0001)
					end


				elseif current.border_top > other.border_top and current.border_top < other.border_bottom then
					-- lower right corner
					margin_x := other.border_right - current.corner_top_left.x
					margin_y := other.border_bottom - current.corner_top_left.y
					if margin_x <= margin_y then
						center.set_x (other.border_right + current.half_width + 0.0001)
						--set_x_y_2 (other.border_right + current.half_width + 0.0001, center.y)
					else
						center.set_y (other.border_bottom + current.half_height + 0.0001)
						--set_x_y_2 (center.x, other.border_bottom + current.half_height + 0.0001)
					end

				end

			end

		end


feature -- Access

	image_offset_x: REAL
			-- The x offset to the EM_RECTANGLE_COLLIDABLE, used for drawing.

	image_offset_y: REAL
			-- The y offset to the EM_RECTANGLE_COLLIDABLE, used for drawing.


end -- class DR_OBJECT
