indexing
	description: "This class represents our hero the dragon."
	author: "Lukas Novosad - novosprotos@eblcom.ch"
	date: "$Date: 2005-06-15 11:55:15 +0200 (Mi, 15 Jun 2005) $"
	revision: "$Revision: 0.8 $"

class
	DRAGON

inherit

	DR_SPRITE_OBJECT
	redefine
		make_from_animation
	end

create
	make_from_animation


feature {NONE} -- Creation

	make_from_animation (an_animation: EM_ANIMATION; an_x, a_y: INTEGER) is
			-- Creates the DRAGON object.
		do
			precursor(an_animation, an_x, a_y)
			set_do_loop(true)
			start
			create direction.make (0.0, 0.0)
			--create last_position.make (0.0, 0.0)
			image_offset_x := 10.0
			image_offset_y := 10.0
			-- Call create from EM_RECTANGLE_COLLIDABLE to ensure the logger gets created (EM_LOGGING class invariant).
			make_from_position_and_size (bounding_box.center, bounding_box.width-2*image_offset_x.floor, bounding_box.height-2*image_offset_y.floor)
		end


feature -- Status setting

	set_hit(a_bool:BOOLEAN) is
			-- Set the hit status.
		do
			hit := a_bool
		ensure
			hit_set: hit = a_bool
		end

	set_direction(a_dir: EM_VECTOR_2D) is
			-- Set the direction the dragon is moving into.
		require a_dir_not_void: a_dir /= void
		do
			direction := a_dir
		ensure
			direction_set: direction = a_dir
		end

--	remember_position is
--			-- Remember the current position.
--		do
--			last_position := last_center
--		ensure
--			last_position_set: last_position = last_center
--		end


feature -- Status


	type_id: INTEGER is
			-- The id given to all instances of `current' generating class.
		do
			result := DRAGON_ID
		end


feature -- Collision handling

	on_collide (other: DR_OBJECT; collision: EM_COLLISION [DR_OBJECT]; frame_delay: INTEGER) is
			-- `current' collided with `other'
		local
			margin_x, margin_y: REAL
			a_stone_moving: STONE_MOVING
		do
			margin_x := 0.0
			margin_y := 0.0

			if other.type_id = ALIEN_ID then
				--io.put_string (time.ticks.out + " DRAGON - on_collide: Dragon collides with an alien.")
				--io.new_line
				set_hit(true)
				other.on_collide (current, collision, frame_delay)

			elseif other.type_id = STONE_BORDER_HORIZONTAL_ID or other.type_id = STONE_BORDER_VERTICAL_ID then
				--io.put_string (time.ticks.out + " DRAGON - on_collide: Dragon collides with a STONE_BORDER_*.")
				--io.new_line
				push_back (other)
				set_x_y ( (corner_top_left.x - image_offset_x).floor, (corner_top_left.y - image_offset_y).floor )

			elseif other.type_id = STONE_MOVING_ID then
				--io.put_string (time.ticks.out + " DRAGON - on_collide: Dragon collides with a STONE_MOVING.")
				--io.new_line
				a_stone_moving ?= other
				if a_stone_moving /= void then
					a_stone_moving.set_pushed (true)
					a_stone_moving.set_pushed_by (current)
					a_stone_moving.set_direction (direction)
					a_stone_moving.step_by (frame_delay)
				end

			elseif other.type_id = STONE_FIXED_ID then
				--io.put_string (time.ticks.out + " DRAGON - on_collide: Dragon collides with a STONE_FIXED.")
				--io.new_line
				push_back (other)
				set_x_y ( (corner_top_left.x - image_offset_x).floor, (corner_top_left.y - image_offset_y).floor )

			elseif other.type_id = POINT_BONUS_ID then
				--io.put_string (time.ticks.out + " DRAGON - on_collide: Dragon collides with a point bonus.")
				--io.new_line
				other.on_collide (current, collision, frame_delay)

			elseif other.type_id = CHEST_ID then
				--io.put_string (time.ticks.out + " DRAGON - on_collide: Dragon collides with a chest.")
				--io.new_line
				other.on_collide (current, collision, frame_delay)

			elseif other.type_id = EXIT_ID then
				--io.put_string (time.ticks.out + " DRAGON - on_collide: Dragon collides with the exit.")
				--io.new_line
				other.on_collide (current, collision, frame_delay)

			end
		end


feature -- Commands

	step_by (a_delay: DOUBLE) is
			-- Move the dragon based on its current direction and the time delay.
		local
			movement: EM_VECTOR_2D
		do
			create movement.make_from_other (direction)
			movement.scale_to (Delta_move_dragon)
			movement.scale (a_delay / 1000)
			--set_center (center + (direction*(a_delay/1000)) )
			--create last_position.make_from_other ( last_center )
			move_by ( movement.x, movement.y )
			set_x_y ( (corner_top_left.x - image_offset_x).floor, (corner_top_left.y - image_offset_y).floor )
		end



feature -- Implementation

	hit: BOOLEAN
		-- Was the DRAGON hit by anything?

	direction: EM_VECTOR_2D
		-- The direction the dragon is currently moving into.

	--last_position: EM_VECTOR_2D
		-- The position before the last call of step_by

end -- class DRAGON
