indexing
	description: "Objects which represent moving stones in the game."
	author: "Lukas Novosad - novosprotos@eblcom.ch"
	date: "$Date: 2005-06-15 11:55:15 +0200 (Mi, 15 Jun 2005) $"
	revision: "$Revision: 0.8 $"

class
	STONE_MOVING

inherit

	DR_SPRITE_OBJECT
	redefine
		--make_from_animation
		make_from_drawable
	end

create
	--make_from_animation
	make_from_drawable


feature {NONE} -- Creation

	make_from_drawable (a_drawable: EM_DRAWABLE; delay, an_x, a_y: INTEGER) is
			-- Create the STONE_MOVING object.
		do
			precursor(a_drawable, delay, an_x, a_y)
			set_do_loop (true)
			start
			set_pushed (false)
			image_offset_x := 0.0
			image_offset_y := 0.0
			--call create from EM_RECTANGLE_COLLIDABLE to ensure the logger gets created (EM_LOGGING class invariant)
			make_from_rectangle(bounding_box)
		end


feature -- Status setting

	set_pushed (b:BOOLEAN) is
			-- Set the push status in the current frame.
		do
			pushed := b
		ensure
			pushed_set: pushed = b
		end


	set_pushed_by (object: DR_OBJECT) is
			-- Set the object that pushed the stone in the current frame.
		do
			pushed_by := object
		ensure
			pushed_by_set: pushed_by = object
		end


	set_direction(a_dir: EM_VECTOR_2D) is
			-- Set the direction the stone is moving into.
		require a_dir_not_void: a_dir /= void
		do
			direction := a_dir
		ensure
			direction_set: direction = a_dir
		end


--	remember_position is
--			-- Remember the current position.
--		do
--			last_position := last_center
--		ensure
--			last_position_set: last_position = last_center
--		end


feature -- Status

	type_id: INTEGER is
			-- The id given to all instances of `current' generating class.
		do
			result := STONE_MOVING_ID;
		end


feature -- Collision handling

	on_collide (other: DR_OBJECT; collision: EM_COLLISION [DR_OBJECT]; frame_delay: INTEGER) is
			-- `current' collided with `other'
		do

			if other.type_id = DRAGON_ID then
				--io.put_string (time.ticks.out + " STONE_MOVING - on_collide: Stone collides with the DRAGON.")
				--io.new_line
					-- Do nothing, this is handled in the ZLEVEL class (handle_movable_stone_collision).

			elseif other.type_id = ALIEN_ID then
				--io.put_string (time.ticks.out + " STONE_MOVING - on_collide: Stone collides with an ALIEN.")
				--io.new_line
				if pushed then
					current.push_back (other)
					current.set_x_y ( (corner_top_left.x - image_offset_x).floor, (corner_top_left.y - image_offset_y).floor )
						-- update the image position
					pushed_by.push_back (current)
				end

			elseif other.type_id = STONE_BORDER_HORIZONTAL_ID or other.type_id = STONE_BORDER_VERTICAL_ID or other.type_id = STONE_FIXED_ID or other.type_id = STONE_MOVING_ID or other.type_id = CHEST_ID or other.type_id = EXIT_ID then
				--io.put_string (time.ticks.out + " STONE_MOVING - on_collide: Stone collides with an obstacle.")
				--io.new_line
				current.push_back (other)
				current.set_x_y ( (corner_top_left.x - image_offset_x).floor, (corner_top_left.y - image_offset_y).floor )
					-- update the image position

				if pushed then
					pushed_by.push_back (current)
				end

			end

		end


feature -- Commands

	step_by (a_delay: DOUBLE) is
			-- Move the stone based on its current direction and the time delay.
		local
			movement: EM_VECTOR_2D
		do
			create movement.make_from_other (direction)
			movement.scale_to (Delta_move_dragon+5)
			movement.scale (a_delay / 1000)
			--set_center (center + (direction*(a_delay/1000)) )
			--create last_position.make_from_other ( last_center )
			move_by ( movement.x, movement.y )
			set_x_y ( (corner_top_left.x - image_offset_x).floor, (corner_top_left.y - image_offset_y).floor )
		end


feature -- Implementation

	pushed: BOOLEAN
		-- Was the stone pushed by another object in the current frame?

	pushed_by: DR_OBJECT
		-- The object that pushed the stone in the current frame.

	direction: EM_VECTOR_2D
		-- The direction the stone is currently moving into.

	--last_position: EM_VECTOR_2D
		-- The position before the last call of step_by.


end -- class STONE_MOVING
