indexing
	description: "Inherit from this class and implement the deferred features if you want to create your own level (see ZLEVEL01)."
	author: "Lukas Novosad - novosprotos@eblcom.ch"
	date: "$Date: 2005-06-15 11:55:15 +0200 (Mi, 15 Jun 2005) $"
	revision: "$Revision: 0.8 $"

deferred class
	ZLEVEL

inherit
	EM_SCENE
	redefine
		handle_key_down_event,
		handle_key_up_event,
		handle_update_event
	end

	EM_SHARED_SUBSYSTEMS

	EM_SHARED_BITMAP_FACTORY

	EM_TIME_SINGLETON

	DR_CONSTANTS


feature -- Initialization

	initialize_scene is
			-- Initialize the scene.
		local

			i: INTEGER

			img: EM_BITMAP
			sbh: STONE_BORDER_HORIZONTAL
			sbv: STONE_BORDER_VERTICAL

			bonus_cursor: DS_LINKED_LIST_CURSOR [BONUS]
			stones_cursor: DS_LINKED_LIST_CURSOR [STONE_MOVING]
			aliens_cursor: DS_LINKED_LIST_CURSOR [ALIEN]
			walls_cursor: DS_LINKED_LIST_CURSOR [STONE_BORDER]

		do

			create_moving_stones

			-- Create the collision detector used for the aliens and the dragon.
			create collision_detector.make (3)
			collision_detector.set_search_depth (0)
			collision_detector.set_movement_check (0)

			-- Create the collision detector for the movable stones.
			if stones_moving /= void then
				create collision_detector_movable_stones.make (stones_moving.count + 1)
			else
				create collision_detector_movable_stones.make (1)
			end
			collision_detector_movable_stones.set_search_depth (0)
			collision_detector_movable_stones.set_movement_check (0)

			-- Create our hero the dragon, collidable set id: 1.
			create_dragon
			collision_detector.add (dragon, 1)
			collision_detector_movable_stones.add (dragon, 1)

			-- Create the level exit, collidable set id: 3.
			create_exit
			collision_detector.add (exit, 3)
			collision_detector_movable_stones.add (exit, 1)

			-- Create the level background.
			background := create_floor

			-- Initialize the keyboard helper array.
			create pressed_keys.make (1, Number_of_keyboard_keys)
			from
				i := 1
			until
				i > Number_of_keyboard_keys
			loop
				pressed_keys.put (False, i)
				i := i + 1
			end

			-- Add the surrounding stone tile patterns, collision set id: 3.
			create walls_border.make
			bitmap_factory.create_bitmap_from_image ("./pics/stonefix.gif")
			img := bitmap_factory.last_bitmap
			img.set_transparent_color (0, 0, 0)
			create sbh.make (img, 12)
			--sbh.set_x_y (0, 0)
			--sbh.move_by (0, 0)
			walls_border.put_last (sbh)
			collision_detector.add (sbh, 3)
			collision_detector_movable_stones.add (sbh, 1)
			create sbv.make (img, 7)
			sbv.set_x_y (0, 85)
			sbv.move_by (0, 85)
			walls_border.put_last (sbv)
			collision_detector.add (sbv, 3)
			collision_detector_movable_stones.add (sbv, 1)
			create sbv.make (img, 7)
			sbv.set_x_y (935, 85)
			sbv.move_by (935, 85)
			walls_border.put_last (sbv)
			collision_detector.add (sbv, 3)
			collision_detector_movable_stones.add (sbv, 1)
			create sbh.make (img, 12)
			sbh.set_x_y (0, 680)
			sbh.move_by (0, 680)
			walls_border.put_last (sbh)
			collision_detector.add (sbh, 3)
			collision_detector_movable_stones.add (sbh, 1)

			-- Create and add interior walls, collision set id: 3.
			create_interior_walls
			if walls_interior /= void then
				create walls_cursor.make (walls_interior)
				from
					walls_cursor.start
				until
					walls_cursor.after
				loop
					collision_detector.add (walls_cursor.item, 3)
					collision_detector_movable_stones.add (walls_cursor.item, 1)
					walls_cursor.forth
				end
			end


			-- Create and add movable stones, collidable set id: 3.
			if stones_moving /= void then
				collision_detector.add_list(stones_moving, 3)

				create stones_cursor.make (stones_moving)
				from
					stones_cursor.start
					i := 2
				until
					stones_cursor.after
				loop
					collision_detector_movable_stones.add (stones_cursor.item, i)
					i := i + 1
					stones_cursor.forth
				end

			end


			-- Create and add boni, collidable set id: 3.
			create_bonus
			if bonus /= void then
				create bonus_cursor.make (bonus)
				from
					bonus_cursor.start
				until
					bonus_cursor.after
				loop
					collision_detector.add (bonus_cursor.item, 3)
					collision_detector_movable_stones.add (bonus_cursor.item, 1)
					bonus_cursor.forth
				end
			end

			-- Create and add aliens, collidable set id: 2.
			create_aliens
			if aliens /= void then
				create aliens_cursor.make (aliens)
				from
					aliens_cursor.start
				until
					aliens_cursor.after
				loop
					collision_detector.add (aliens_cursor.item, 2)
					collision_detector_movable_stones.add (aliens_cursor.item, 1)
					aliens_cursor.forth
				end
			end

		    -- Create the lives_icons.
			create lives_icons.make (1, Max_number_of_lives - 1)
			from
				i := lives_icons.lower
			until
				i = Max_number_of_lives
			loop
				bitmap_factory.create_bitmap_from_image("./pics/dragon01.gif")
				img := bitmap_factory.last_bitmap
				lives_icons.put (img, i)
				lives_icons.item (i).set_x_y (0 + (i-1) * (lives_icons.item (i).width), 780)
				i := i + 1
			end

			-- Create the score string.
			bitmap_factory.create_bitmap_from_image ("./pics/arial256.gif")
			create score_str.make ("score: "+score.out,create {EM_BMP_FONT}.make(bitmap_factory.last_bitmap))
			score_str.set_x_y (0, 870)

			-- Create the frame counter.
			create frame_counter.make
			frame_counter.set_x_y (870,870)

			is_paused := false

			last_frame := time.ticks
			frame_delay := 0

			-- Hide the mouse cursor.
			video_subsystem.hide_cursor

		end


feature -- Commands

	create_floor:EM_BITMAP is
		-- Choose the picture used for the floor in the level.
		deferred
	end


	create_dragon is
		-- Create the dragon and position it in the level.
		deferred
	end


	create_interior_walls is
		-- Create the walls for the level.
		deferred
	end


	create_moving_stones is
		-- Create the movable stones for the level.
		deferred
	end


	create_bonus is
		-- Create bonus objects for the level.
		deferred
	end


	create_aliens is
		-- Create the aliens for the level.
		deferred
	end


	create_exit is
		-- Create the level exit and set its position in the level.
		deferred
		ensure
			--xLoc_set: exit.x > 0 and exit.x < 1024
			xLoc_set: exit.center.x > 0 and exit.center.x < 1024
			--yLoc_set: exit.y > 0 and exit.y < 768
			yLoc_set: exit.center.y > 0 and exit.center.y < 768
		end


feature -- Status setting

	set_score (s: INTEGER) is
			-- Set `score' to `s'
		do
			score := s
		ensure
			score_set: score = s
		end

	set_lives (i: INTEGER) is
			-- Set `lives' to `i'
		do
			lives := i
		ensure
			lives_set: lives = i
		end

	toggle_paused is
			-- Toggle `is_paused'
		do
			is_paused := not is_paused
			time.set_are_timed_procedures_paused (is_paused)
		ensure
			paused_toggled: is_paused = not old is_paused
		end


feature {NONE} -- Event handler

	handle_key_down_event (a_keyboard_event: EM_KEYBOARD_EVENT) is
		-- Handle the keyboard key down events.
		require else
			a_keyboard_event_not_void: a_keyboard_event /= Void
		do
			if a_keyboard_event.key = a_keyboard_event.sdlk_f then
				frame_counter.set_visible (not frame_counter.is_visible)

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_p then
				toggle_paused

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_c then
				cheat := not cheat

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_l and lives < Max_number_of_lives then
				set_lives (lives + 1)

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_n then
				event_loop.stop
				on_win

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_escape then
				event_loop.stop
				if next_scene = void then
					set_next_scene (create{START}.make)
				end

			-- Allow the following keyboard inputs only if game is not paused.
			elseif not is_paused then

				if a_keyboard_event.key = a_keyboard_event.sdlk_up then
					pressed_keys.put (True, UP)

				elseif a_keyboard_event.key = a_keyboard_event.sdlk_down then
					pressed_keys.put (True, DOWN)

				elseif a_keyboard_event.key = a_keyboard_event.sdlk_right then
					pressed_keys.put (True, RIGHT)

				elseif a_keyboard_event.key = a_keyboard_event.sdlk_left then
					pressed_keys.put (True, LEFT)
				end

			end
		end


	handle_key_up_event (a_keyboard_event: EM_KEYBOARD_EVENT) is
			-- Handle the keyboard key up events.
		require else
			a_keyboard_event_not_void: a_keyboard_event /= Void
		do
			if a_keyboard_event.key = a_keyboard_event.sdlk_up then
				pressed_keys.put (False, UP)

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_down then
				pressed_keys.put (False, DOWN)

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_right then
				pressed_keys.put (False, RIGHT)

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_left then
				pressed_keys.put (False, LEFT)

			end
		end


	redraw is
			-- Redraw the whole scene.
		local
			i: INTEGER
			bonus_cursor: DS_LINKED_LIST_CURSOR [BONUS]
			aliens_cursor: DS_LINKED_LIST_CURSOR [ALIEN]
			stones_cursor: DS_LINKED_LIST_CURSOR [STONE_MOVING]
			walls_cursor: DS_LINKED_LIST_CURSOR [STONE_BORDER]
		do

			screen.clear

			-- Draw the level background.
			background.draw (screen)

			-- Draw the score string.
			score_str.draw (screen)

			-- Draw the framecounter.
			frame_counter.draw (screen)

			-- Draw all live icons.
			from
				i := lives_icons.lower
			until
				i = lives
			loop
				lives_icons.item (i).draw (screen)
				i := i + 1
			end

			-- Draw the dragon (hero).
			dragon.draw (screen)

			-- Draw the border walls.
			if walls_border /= void then
				create walls_cursor.make (walls_border)
				from
					walls_cursor.start
				until
					walls_cursor.after
				loop
					walls_cursor.item.draw (screen)
					walls_cursor.forth
				end
			end

			-- Draw all interior walls.
			if walls_interior /= void then
				create walls_cursor.make (walls_interior)
				from
					walls_cursor.start
				until
					walls_cursor.after
				loop
					walls_cursor.item.draw (screen)
					walls_cursor.forth
				end
			end

			-- Draw all movable stones.
			if stones_moving /= void then
				create stones_cursor.make (stones_moving)
				from
					stones_cursor.start
				until
					stones_cursor.after
				loop
					stones_cursor.item.draw (screen)
					stones_cursor.forth
				end
			end

			-- Draw all boni which have not been collected yet.
			if bonus /= void then
				create bonus_cursor.make (bonus)
				from
					bonus_cursor.start
				until
					bonus_cursor.after
				loop
					bonus_cursor.item.draw (screen)
					bonus_cursor.forth
				end
			end

			-- Draw all aliens which are still alive.
			if aliens /= void then
				create aliens_cursor.make (aliens)
				from
					aliens_cursor.start
				until
					aliens_cursor.after
				loop
					if not (aliens_cursor.item.state = -1) then
						aliens_cursor.item.draw (screen)
					end
					aliens_cursor.forth
				end
			end

			-- Draw the level exit.
			exit.draw (screen)

			-- Switch the drawing buffers (front/back).
			screen.redraw

		end


	handle_update_event is
			-- Handle the outside event.
			-- This includes the object movements in the involved physics therein.
		local
			current_time: INTEGER
			bonus_cursor: DS_LINKED_LIST_CURSOR [BONUS]
			aliens_cursor: DS_LINKED_LIST_CURSOR [ALIEN]
			stones_cursor: DS_LINKED_LIST_CURSOR [STONE_MOVING]
			a_chest: CHEST
			a_point_bonus: POINT_BONUS
		do

			-- Calculate the frame delay, e.g. the elapsed time since the last frame.
			frame_delay := time.ticks - last_frame

			-- Slow down if framerate drops too much.
			if frame_delay >= Max_frame_delay then
				frame_delay := Max_frame_delay
				current_time := time.ticks
			end

			-- Reset the pusher objects on all movable stones.
			if stones_moving /= void then
				create stones_cursor.make (stones_moving)
				from
					stones_cursor.start
				until
					stones_cursor.after
				loop
					stones_cursor.item.set_pushed (false)
					stones_cursor.item.set_pushed_by (void)
					stones_cursor.forth
				end
			end


			--io.put_string (time.ticks.out + " ZLEVEL - handle_update_event: New frame with a frame_delay of " + frame_delay.out + "ms")
			--io.new_line

			-- Move all objects and check for collisions.
			-- Do this only while the game is running and not paused.
			if not is_paused then

				-- Move the aliens.
				move_aliens

				-- Move the dragon.
				move_dragon

				-- Handle all collisions.
				handle_collisions

				-- Check if the dragon was hit.
				if dragon.hit then
					if lives = 1 then
						on_lose
					else
						lives := lives - 1
						dragon.set_hit(false)
					end
				end


				-- Check for collected boni.
				if bonus /= void then
					create bonus_cursor.make (bonus)
					from
						bonus_cursor.start
					until
						bonus_cursor.after
					loop
						if bonus_cursor.item.collected then
							a_point_bonus ?= bonus_cursor.item
							if a_point_bonus /= void then
								score := score + bonus_cursor.item.bonus
								score_str.set_value ("score: " + score.out)
								--io.put_string (time.ticks.out + " ZLEVEL - handle_update_event: detected a collected point bonus with value "+a_point_bonus.bonus.out)
								--io.new_line
								collision_detector.remove (a_point_bonus)
								collision_detector_movable_stones.remove (a_point_bonus)
								bonus_cursor.remove
							else
								a_chest ?= bonus_cursor.item
								if a_chest /= void then
									if a_chest.closed then
										--io.put_string (time.ticks.out + " ZLEVEL - handle_update_event: detected a collected chest with value "+a_chest.bonus.out)
										--io.new_line
										score := score + bonus_cursor.item.bonus
										score_str.set_value ("score: " + score.out)
										a_chest.set_closed (false)
										collision_detector.remove (a_chest)
										collision_detector_movable_stones.remove (a_chest)
									end
									bonus_cursor.forth
								end
							end

						else
							bonus_cursor.forth
						end
					end
				end

				-- Check for dead aliens.
				if aliens /= void then
					create aliens_cursor.make (aliens)
					from
						aliens_cursor.start
					until
						aliens_cursor.after
					loop
						if (aliens_cursor.item.state = DEAD) then
							collision_detector.remove (aliens_cursor.item)
							collision_detector_movable_stones.remove (aliens_cursor.item)
							aliens_cursor.remove
						else
							aliens_cursor.forth
						end
					end

				end

				-- Check if the dragon has reached the exit.
				if exit.arrived then
					event_loop.stop
					on_win
				end

			end

			-- Remember the time of the last frame.
			-- If the frame took too long, set it to the current system time.
			last_frame := last_frame + frame_delay
			if frame_delay >= Max_frame_delay then
				last_frame := current_time
			end


		end


feature -- Procedures

	move_aliens is
		-- Describes the finite state machines of the aliens, must be implemented by client, e.g. the effective level class.
		deferred
	end


	move_dragon is
			-- Move the dragon (hero)
		local
			dir: EM_VECTOR_2D
		do
			create dir.make (0.0, 0.0)
				--the dragon's moving direction

			if pressed_keys.item (UP) then
				dir.add ( create {EM_VECTOR_2D}.make (0.0, -1.0) )
			end

			if pressed_keys.item (LEFT) then
				dir.add ( create {EM_VECTOR_2D}.make (-1.0, 0.0) )
			end

			if pressed_keys.item (RIGHT) then
				dir.add ( create {EM_VECTOR_2D}.make (1.0, 0.0) )
			end

			if pressed_keys.item (DOWN) then
				dir.add ( create {EM_VECTOR_2D}.make (0.0, 1.0) )
			end

			--io.put_string (time.ticks.out + " ZLEVEL - move_dragon: The dragon will be moving into ("+dir.x.out+","+dir.y.out+")")
			--io.new_line
			dragon.set_direction (dir)
			--dragon.remember_position
			if dir.length > 0.0 then
				-- Only move the dragon if there was any user input {UP, DOWN, LEFT, RIGHT}.
				dragon.step_by (frame_delay)
			end
		end


	on_win is
			-- Called if player won, set next scene here.
		deferred
		end


	on_lose is
			-- Called if player lost, return to the Start menu.
		do
			event_loop.stop
			if next_scene = void then
				set_next_scene(create{START}.make)
				start_next_scene
			end
		end


feature -- Collision Handling

	handle_collisions is
			-- The main collision handling routine calls subroutines if needed.
		local
			composition_cursor: DS_LINKED_LIST_CURSOR [EM_COLLISION_COMPOSITION [DR_OBJECT]]
			collision_cursor: DS_LINKED_LIST_CURSOR [EM_COLLISION [DR_OBJECT]]
			collision: EM_COLLISION [DR_OBJECT]
		do

			-- check for dragon and alien collisions using the EM_COLLISION_DETECTOR collision_detector
			collision_detector.check_for_collision
			--io.put_string ("ZLEVEL-handle_collisions: checked for collisions")
			--io.new_line

			-- get the last collsision in form of an EM_COLLISION_COMPOSITION of EM_RECTANGLE_COLLIDABLE objects
			create composition_cursor.make (collision_detector.last_collisions)

			-- TODO: optimize with last_collisions.has_collidable

			-- loop through all collision compositions
			from
				composition_cursor.start
			until
				composition_cursor.after
			loop
				create collision_cursor.make (composition_cursor.item)
				-- loop through all single collisions
				from
					collision_cursor.start
				until
					collision_cursor.after
				loop
					collision := collision_cursor.item
					if collision.collision_point /= void then
						-- handle elements which intersect without inclusion
						--if is_dragon_collision then
						if collision.collidables.first.type_id = DRAGON_ID or collision.collidables.second.type_id = DRAGON_ID then
							--io.put_string (time.ticks.out + " ZLEVEL - handle_collisions: detected dragon collision with col. time "+collision.time.out+"ms")
							--io.new_line
							handle_dragon_collision (collision_cursor.item)
						--elseif is_alien_collision (collision_cursor.item) then
						elseif collision.collidables.first.type_id = ALIEN_ID or collision.collidables.second.type_id = ALIEN_ID then
							--io.put_string (time.ticks.out + " ZLEVEL - handle_collisions: detected alien collision with col. time "+collision.time.out+"ms")
							--io.new_line
							handle_alien_collision (collision_cursor.item)
						end
					else
						-- TODO: handle elements which intersect by inclusion
					end

					collision_cursor.forth
				end

				composition_cursor.forth
			end

			-- check for movable stone collisions using the EM_COLLISION_DETECTOR collision_detector_movable_stones
			collision_detector_movable_stones.check_for_collision
			--io.put_string ("ZLEVEL-handle_collisions: checked for movable stones collisions")
			--io.new_line

			-- get the last collsision in form of an EM_COLLISION_COMPOSITION of EM_RECTANGLE_COLLIDABLE objects
			create composition_cursor.make (collision_detector_movable_stones.last_collisions)

			-- TODO: optimize with last_collisions.has_collidable

			-- loop through all collision compositions
			from
				composition_cursor.start
			until
				composition_cursor.after
			loop
				create collision_cursor.make (composition_cursor.item)
				-- loop through all single collisions
				from
					collision_cursor.start
				until
					collision_cursor.after
				loop
					collision := collision_cursor.item
					if collision.collision_point /= void then
						-- handle elements which intersect without inclusion
						--if is_movable_stone_collision then
						--if collision.collidables.first.type_id = STONE_MOVING_ID or collision.collidables.second.type_id = STONE_MOVING_ID then
							--io.put_string (time.ticks.out + " ZLEVEL - handle_collisions: detected movable stone collision with col. time "+collision.time.out+"ms")
							--io.new_line
							handle_movable_stone_collision (collision_cursor.item)
						--else

						--end
					else
						-- handle elements which intersect by inclusion
					end

					collision_cursor.forth
				end

				composition_cursor.forth
			end

		end


	handle_dragon_collision (collision: EM_COLLISION [DR_OBJECT] )is
			-- Handle a dragon collision.
		local
		do
			if collision.collidables.first.type_id = DRAGON_ID then
				collision.collidables.first.on_collide(collision.collidables.second, collision, frame_delay)
			else
				collision.collidables.second.on_collide(collision.collidables.first, collision, frame_delay)
			end
		end


	handle_alien_collision (collision: EM_COLLISION [DR_OBJECT] ) is
			-- Handle an alien collision.
		local
		do
			if collision.collidables.first.type_id = ALIEN_ID then
				collision.collidables.first.on_collide(collision.collidables.second, collision, frame_delay)
			else
				collision.collidables.second.on_collide(collision.collidables.first, collision, frame_delay)
			end
		end

	handle_movable_stone_collision (collision: EM_COLLISION [DR_OBJECT] ) is
				-- Handle a movable stone collision.
		local
		do
			if collision.collidables.first.type_id = STONE_MOVING_ID then
				collision.collidables.first.on_collide(collision.collidables.second, collision, frame_delay)
			else
				collision.collidables.second.on_collide(collision.collidables.first, collision, frame_delay)
			end
		end



feature {NONE} -- Implementation

	dragon: DRAGON
		-- our hero the dragon

	background: EM_BITMAP
		-- The background used for the level.

	collision_detector: EM_COLLISION_DETECTOR [DR_OBJECT]
		-- The collision detector used for the dragon and the aliens.

	collision_detector_movable_stones: EM_COLLISION_DETECTOR [DR_OBJECT]
		-- The collision detector used for the movable stones.

	cheat: BOOLEAN
		-- Is the cheat mode on?

	last_frame: INTEGER
		-- the time stamp of the last frame

	frame_delay: INTEGER
		-- the delay between the last and the current frame

	lives_icons: ARRAY [EM_BITMAP]
		-- lives icons

	bonus: DS_LINKED_LIST [BONUS]
		-- All the bonuses in the level

	aliens: DS_LINKED_LIST [ALIEN]
		-- All the aliens in the level

	stones_moving: DS_LINKED_LIST [STONE_MOVING]
		-- all the moving stones in the level

	stones_fixed: DS_LINKED_LIST [STONE_FIXED]
		-- All the fixed stones in the level

	walls_border: DS_LINKED_LIST [STONE_BORDER]
		-- All the level border walls

	walls_interior: DS_LINKED_LIST [STONE_BORDER]
		-- Used for additional walls

	lives: INTEGER
		-- number of lives left

	score: INTEGER
		-- the current score

	score_str: EM_STRING
		-- the score string

	exit: EXIT
		-- the exit for the dragon

	is_paused: BOOLEAN
		-- Is the game paused?

	pressed_keys: ARRAY [BOOLEAN]
		-- The array indicating the currently pressed keyboard keys.


end -- class ZLEVEL
