indexing
	description: "MENU contains functionalities used by all menus in the game."
	author: "Lukas Novosad - novosprotos@eblcom.ch"
	date: "$Date: 2005-06-15 11:55:15 +0200 (Mi, 15 Jun 2005) $"
	revision: "$Revision: 0.8 $"

deferred class

	MENU

inherit

	EM_WIDGET_SCENE
	redefine
		handle_key_down_event
	end

	EM_SHARED_STANDARD_FONTS
	export
		{NONE} all
	end

	DR_CONSTANTS


feature {NONE} -- Creation

	initialize_menu is
			-- Create the menu.
		do
			make_widget_scene

			-- create background
			create menu_background.make_from_file("./pics/dr_menu.png")
			set_background (menu_background)

			-- create title
--			create title.make_from_text ("Dragoony")
--			bitmap_factory.create_bitmap_from_image("./pics/arial256.gif")
--			title.set_font(create {EM_BMP_FONT}.make(bitmap_factory.last_bitmap))
--			title.set_foreground_color (create {EM_COLOR}.make_white)
--			title.resize_to_optimal_dimension
--			title.set_position (width // 2 - title.width // 2, 10)
--			add_widget(title)

			-- create version number
			create version.make_from_text ("V 0.8")
			version.set_font(standard_ttf_fonts.bitstream_vera_sans (15))
			version.set_foreground_color (create {EM_COLOR}.make_white)
			version.resize_to_optimal_dimension
			version.set_position (0, height - version.height)
			add_widget(version)

			create menu_buttons.make

			video_subsystem.show_cursor

		end


feature -- Status setting

	set_selected_menu_item (item: integer) is
			-- select menu item 'item'
		require
			valid_menu_item: item >= 1 and item <= menu_buttons.count
		do
			if selected_menu_item >= 1 and selected_menu_item <= menu_buttons.count then
				menu_buttons.i_th (selected_menu_item).set_foreground_color (create {EM_COLOR}.make_white)
			end
			menu_buttons.i_th (item).set_foreground_color (create {EM_COLOR}.make_with_rgb(255,0,0))
			selected_menu_item := item
		ensure
			selected_menu_item_set: selected_menu_item = item
		end


feature -- Procedures

	add_button (a_button: MENU_BUTTON) is
			-- Adds `button' to the menu.
		require
			a_button_not_void: a_button /= Void
		do
			a_button.mouse_entered_event.subscribe (agent menu_item_mouse_enter (menu_buttons.count + 1))
			a_button.pressed_event.subscribe (agent handle_pressed_event)
			menu_buttons.extend (a_button)
			add_widget (a_button)
		end


feature {NONE} -- Event handler

	back_button_pressed is
			-- back button pressed
		deferred
		end


	handle_pressed_event is
			-- Handle pressed events.
		do
		end


	menu_item_mouse_enter (item: INTEGER) is
			-- Handle mouse enter events.
		require
			valid_menu_item: item >= 1 and item <= menu_buttons.count
		do
			set_selected_menu_item(item)
		end


	handle_key_down_event (a_keyboard_event: EM_KEYBOARD_EVENT) is
			-- Handle keyboard key down events.
		require else
			a_keyboard_event_not_void: a_keyboard_event /= Void
		do
			Precursor (a_keyboard_event)

			if a_keyboard_event.key = a_keyboard_event.sdlk_down then
				if selected_menu_item < menu_buttons.count then
					set_selected_menu_item (selected_menu_item + 1)
				end

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_up then
				if selected_menu_item > 1 then
					set_selected_menu_item (selected_menu_item - 1)
				end

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_return then
				menu_buttons.i_th (selected_menu_item).pressed_event.publish ([])

			elseif a_keyboard_event.key = a_keyboard_event.sdlk_escape then
				back_button_pressed
			end

		end


feature -- Status

	menu_buttons: LINKED_LIST[MENU_BUTTON]
		-- all menu items/buttons


feature {NONE} -- Implementation


	menu_background: EM_BITMAP_BACKGROUND
		-- menu background bitmap

	title: EM_LABEL
		-- menu title

	version: EM_LABEL
		-- version number

	select_font: EM_BMP_FONT
		-- The font used to display a selected menu entry.

	normal_font: EM_BMP_FONT
		-- The font used to display the menu entries.

	selected_menu_item: INTEGER
		-- selected menu item


end -- class MENU
